import bpy
from bpy.props import (BoolProperty, FloatProperty)


#=====================================================
#          Set Material
#=====================================================



class OBJECT_OT_set_default_material_anbx(bpy.types.Operator):
    '''Create and assign a new Individual Material(s) - if not exists 
- to every Selected Object(s)
- Principled BSDF'''


    bl_idname = "object.set_default_material_anbx"
    bl_label = "Set Material"
    bl_options = {'REGISTER', 'UNDO'}


    @classmethod
    def poll(cls, context):

        object_types = ['CURVE', 'MESH', 'FONT', 'SURFACE', 'META']

        return (context.object is not None and
                context.object.mode == 'OBJECT' and
                len(context.selected_objects) != 0 and
                context.object.type in object_types)


    def execute(self, context):

        mat_object_types = ['CURVE', 'MESH', 'FONT', 'SURFACE', 'META']

        sel_ob = bpy.context.selected_objects
        act_ob = bpy.context.active_object


        # Add Material if not Exist


        def add_new_mat_to_active():
            mat = act_ob.data.materials
            if not mat:
                new_mat = bpy.data.materials.new(act_ob.name + '_mat')
                act_ob.data.materials.append(new_mat)
                bpy.context.object.active_material.use_nodes = True


        def add_new_mat_to_multi_selected():
            for o in sel_ob:
                bpy.context.view_layer.objects.active = o

                if o.type in mat_object_types:

                    mat = o.data.materials
                    if not mat:
                        new_mat = bpy.data.materials.new(o.name + '_mat')
                        o.data.materials.append(new_mat)
                        bpy.context.object.active_material.use_nodes = True
                        bpy.ops.object.make_single_user(object=False, obdata=False, material=True, animation=False)


        #---------------------------------------

        if len(sel_ob) == 1:
            add_new_mat_to_active()


        if len(sel_ob) > 1:
            add_new_mat_to_multi_selected()
            bpy.ops.object.make_single_user(object=False, obdata=False, material=True, animation=False)


 

        return {'FINISHED'}


#=====================================================
#          End  Set Material
#=====================================================



#====================== Material Transparency ========
 
def change_alpha_value_fun_anbx(self, context):
 
    sel_ob = bpy.context.selected_objects
    act_ob = bpy.context.active_object
    cur_frame = bpy.context.scene.frame_current

    add_keyframe = context.scene.mat_light_add_keyframe_anbx    

 
    for o in sel_ob:
        mat_object_types = ['CURVE', 'MESH', 'FONT', 'SURFACE', 'META']
        bpy.context.view_layer.objects.active = o
        
        # Setting
        o.active_material.use_backface_culling = True
        o.active_material.blend_method = 'BLEND'
        o.active_material.shadow_method = 'HASHED'

        if o.type in mat_object_types:

            for m in o.data.materials:
                mat_name = m.name

                num = 19
                # Check Blender Version
                blender_version = bpy.app.version[0]

                if blender_version == 2:
                    num = 19

                if blender_version == 3:
                    num = 21

                if blender_version == 4:
                    num = 4

                # Alpha
                for n in bpy.data.materials[mat_name].node_tree.nodes:
                    if n.name == "Principled BSDF":
                        n.inputs[num].default_value  = self.change_alpha_value_anbx

                        if add_keyframe:
                            n.inputs[num].keyframe_insert(data_path="default_value", frame=cur_frame)


#====================== Material Transparency ========
 




classes = [OBJECT_OT_set_default_material_anbx]





def register():
    for cls in classes:
        bpy.utils.register_class(cls)



    bpy.types.Scene.change_alpha_value_anbx = bpy.props.FloatProperty(
            name="Alpha",
            default=1.0,
            precision=3,
            step=10,
            min=0.000, max=1.000,
            description="Alpha Value - Principled BSDF",
            update = change_alpha_value_fun_anbx
            )




    # If checked = add keyframe to material color, alpha, light color-power
    bpy.types.Scene.mat_light_add_keyframe_anbx = bpy.props.BoolProperty(
        name="",
        description="Add Keyframe Option - Material, Light",
        default = False
    )


def unregister():
    for cls in classes:
        bpy.utils.unregister_class(cls)

    del bpy.types.Scene.change_alpha_value_anbx
    del bpy.types.Scene.mat_light_add_keyframe_anbx


if __name__ == "__main__":
    register()






